/** 
 @name DistributeLayers
 @version 2.0 (05 June 2009)
 @fileoverview
 <h4>Description</h4>  
 <p>This script allows you to distribute the selected layers in 3D space.
 In addition to offset position, you can offset rotation/scale/opacity and add some randomness.
 The "Factor" parameter allows non-linear offset between layers.</p> 
 <h4>Usage</h4>
 <ol>
    <li> In After Effects CS4 or later, run the script
    <li> Select at least two layers  
    <li> Select a property in the tabbed panel 
    <li> Drag slider or edit text input field to see the changes in the active comp
 </ol>
 @author Charles Bordenave
 @location http://www.nabscripts.com/downloads/scripts/DistributeLayers.zip
*/


/**
 This class represents the main class of the script;
 it is used to create the user interface which allows to distribute layers in 3D space
 @class Main class of the script
*/
function DistributeLayers()
{
    // Variable used to keep track of 'this' reference
    var distributeLayers = this;
    
    // Create an instance of the utils class to use its functions
    var utils = new DistributeLayersUtils();

    // Script infos
    this.scriptMinSupportVersion = "9.0";
    this.scriptName = "DistributeLayers.jsx";    
    this.scriptVersion = "2.0";
    this.scriptTitle = "Distribute Layers";
    this.scriptCopyright = "Copyright (c) 2009 Charles Bordenave";
    this.scriptHomepage = "http://www.nabscripts.com";
    this.scriptDescription = {en: "This script allows you to distribute the selected layers in 3D space.\\r\\rIn addition to offset position, you can offset rotation/scale/opacity and add some randomness.\\r\\rThe \"Factor\" parameter allows non-linear offset between layers.", fr:"Ce script permet de distribuer les calques dans l\\'espace 3D.\\r\\rEn plus de pouvoir décaler la position, vous pouvez décaler la rotation/l\\'échelle/l\\'opacité et ajouter de l\\'aléatoire.\\r\\rLe paramètre \" Facteur\" permet d\\'obtenir un décalage non-linéaire entre les calques."};
    this.scriptAbout = {en:this.scriptName + ", v" + this.scriptVersion + "\\r\\r" + this.scriptCopyright + "\\r" + this.scriptHomepage + "\\r\\r" + utils.loc(this.scriptDescription), fr:this.scriptName + ", v" + this.scriptVersion + "\\r\\r" + this.scriptCopyright + "\\r" + this.scriptHomepage + "\\r\\r" + utils.loc(this.scriptDescription)};        

    // Errors
    this.requirementErr = {en:"This script requires After Effects CS4 or later.", fr:"Ce script nécessite After Effects CS4 ou supérieur."};    
    this.noCompErr = {en:"A comp must be active.", fr:"Une composition doit être active."};
    this.noLayersErr = {en:"Select at least two layers.", fr:"Sélectionnez au moins deux calques."};
    this.duringProcessErr = {en:"An error occurred while accessing transform properties of selected layers.", fr:"Une erreur s'est produite lors de l'accès aux propriétés de transformation des calques sélectionnés."};

    // UI strings 
    this.aboutBtnName = "?";
    this.runBtnName = {en:"Run", fr:"Éxécuter"};
    
    // Miscellaneous
    //this.sclWidth =     
    /**
     Creates and displays the script interface
     @param {Object} thisObj A Panel object if the script is launched from the Window menu, null otherwise    
    */
    this.buildUI = function (thisObj)
    {
        // dockable panel or palette
        var pal = (thisObj instanceof Panel) ? thisObj : new Window("palette", this.scriptTitle, undefined, {resizeable:true});

        // resource specifications
        var res =
        "group { orientation:'column', alignment:['fill','top'], alignChildren:['right','top'], \
            gr1: Group { alignment:['fill','fill'], \
                aboutBtn: Button { text:'?', preferredSize:[25,20], alignment:['right','center'] } \
            }, \
            pnl: Panel { type:'tabbedpanel', alignment:['fill','fill'], \
                pnl1: Panel { type:'tab', text:'Pos', orientation:'row', alignment:['fill','fill'], alignChildren:['right','top'], \
                    gr1: Group { orientation:'column', alignment:['left','top'], alignChildren:['right','center'], \
                           xOffsetSt: StaticText { text:'X Offset:', preferredSize:[-1,18] }, \
                        yOffsetSt: StaticText { text:'Y Offset:', preferredSize:[-1,18] }, \
                        zOffsetSt: StaticText { text:'Z Offset:' } \
                    }, \
                    gr2: Group { orientation:'column', alignment:['fill','top'], \
                        gr21: Group { orientation:'row', alignment:['fill','top'], \
                            xOffsetScl: Scrollbar { minvalue:-100, alignment:['fill','center'] }, \
                            xOffsetEt: EditText { text:'0', characters:5, alignment:['right','center'] } \
                        }, \
                        gr22: Group { orientation:'row', alignment:['fill','top'], \
                            yOffsetScl: Scrollbar { minvalue:-100, alignment:['fill','center'] }, \
                            yOffsetEt: EditText { text:'0', characters:5, alignment:['right','center'] } \
                        }, \
                        gr23: Group { orientation:'row', alignment:['fill','top'], \
                            zOffsetScl: Scrollbar { minvalue:-100, alignment:['fill','center'] }, \
                            zOffsetEt: EditText { text:'0', characters:5, alignment:['right','center'] } \
                        }, \
                        gr24: Group { alignment:'fill', alignChildren:['left','top'], \
                            sliderRangeBtn: Button { text:'Slider Range' } \
                        } \
                    } \
                }, \
                pnl2: Panel { type:'tab', text:'Rot', orientation:'row', alignment:['fill','fill'], alignChildren:['right','top'], \
                    gr1: Group { orientation:'column', alignment:['left','top'], alignChildren:['right','top'], \
                           xRotationSt: StaticText { text:'X Rotation:', preferredSize:[-1,18] }, \
                        yRotationSt: StaticText { text:'Y Rotation:', preferredSize:[-1,18] }, \
                        zRotationSt: StaticText { text:'Z Rotation:' } \
                    }, \
                    gr2: Group { orientation:'column', alignment:['fill','top'], \
                        gr21: Group { orientation:'row', alignment:['fill','top'], \
                            xRotationScl: Scrollbar { minvalue:-100, alignment:['fill','center'] }, \
                            xRotationEt: EditText { text:'0', characters:5, alignment:['right','center'] } \
                        }, \
                        gr22: Group { orientation:'row', alignment:['fill','top'], \
                            yRotationScl: Scrollbar { minvalue:-100, alignment:['fill','center'] }, \
                            yRotationEt: EditText { text:'0', characters:5, alignment:['right','center'] } \
                        }, \
                        gr23: Group { orientation:'row', alignment:['fill','top'], \
                            zRotationScl: Scrollbar { minvalue:-100, alignment:['fill','center'] }, \
                            zRotationEt: EditText { text:'0', characters:5, alignment:['right','center'] } \
                        }, \
                        gr24: Group { alignment:'fill', alignChildren:['left','top'], \
                            sliderRangeBtn: Button { text:'Slider Range' } \
                        } \
                    } \
                }, \
                pnl3: Panel { type:'tab', text:'Sca/Opa', orientation:'row', alignment:['fill','fill'], alignChildren:['right','top'], \
                    gr1: Group { orientation:'column', alignment:['left','top'], alignChildren:['right','top'], \
                           xScaleSt: StaticText { text:'X Scale Offset:', preferredSize:[-1,18] }, \
                           yScaleSt: StaticText { text:'Y Scale Offset:', preferredSize:[-1,18] }, \
                        opacitySt: StaticText { text:'Opacity Offset:', preferredSize:[-1,18] }, \
                        fooSt: StaticText { visible:false, text:'foo:' } \
                    }, \
                    gr2: Group { orientation:'column', alignment:['fill','top'], \
                        gr21: Group { orientation:'row', alignment:['fill','top'], \
                            xScaleScl: Scrollbar { minvalue:-100, alignment:['fill','center'] }, \
                            xScaleEt: EditText { text:'0', characters:5, alignment:['right','center'] } \
                        }, \
                        gr22: Group { orientation:'row', alignment:['fill','top'], \
                            yScaleScl: Scrollbar { minvalue:-100, alignment:['fill','center'] }, \
                            yScaleEt: EditText { text:'0', characters:5, alignment:['right','center'] } \
                        }, \
                        gr23: Group { orientation:'row', alignment:['fill','top'], \
                            opacityScl: Scrollbar { alignment:['fill','center'] }, \
                            opacityEt: EditText { text:'0', characters:5, alignment:['right','center'] } \
                        }, \
                        gr24: Group { alignment:'fill', alignChildren:['left','top'], \
                            sliderRangeBtn: Button { text:'Slider Range' } \
                        } \
                    } \
                }, \
                pnl4: Panel { type:'tab', text:'Rand Pos', orientation:'row', alignment:['fill','fill'], alignChildren:['right','top'], \
                    gr1: Group { orientation:'column', alignment:['left','top'], alignChildren:['right','top'], \
                           xRandomSt: StaticText { text:'X Random:', preferredSize:[-1,18] }, \
                        yRandomSt: StaticText { text:'Y Random:', preferredSize:[-1,18] }, \
                        zRandomSt: StaticText { text:'Z Random:' } \
                    }, \
                    gr2: Group { orientation:'column', alignment:['fill','top'], \
                        gr21: Group { orientation:'row', alignment:['fill','top'], \
                            xRandomScl: Scrollbar { alignment:['fill','center'] }, \
                            xRandomEt: EditText { text:'0', characters:5, alignment:['right','center'] } \
                        }, \
                        gr22: Group { orientation:'row', alignment:['fill','top'], \
                            yRandomScl: Scrollbar { alignment:['fill','center'] }, \
                            yRandomEt: EditText { text:'0', characters:5, alignment:['right','center'] } \
                        }, \
                        gr23: Group { orientation:'row', alignment:['fill','top'], \
                            zRandomScl: Scrollbar { alignment:['fill','center'] }, \
                            zRandomEt: EditText { text:'0', characters:5, alignment:['right','center'] } \
                        }, \
                        gr24: Group { alignment:'fill', alignChildren:['left','top'], \
                            sliderRangeBtn: Button { text:'Slider Range' } \
                        } \
                    } \
                }, \
                pnl5: Panel { type:'tab', text:'Rand Rot', orientation:'row', alignment:['fill','fill'], alignChildren:['right','top'], \
                    gr1: Group { orientation:'column', alignment:['left','top'], alignChildren:['right','top'], \
                           xRandomSt: StaticText { text:'X Random:', preferredSize:[-1,18] }, \
                        yRandomSt: StaticText { text:'Y Random:', preferredSize:[-1,18] }, \
                        zRandomSt: StaticText { text:'Z Random:' } \
                    }, \
                    gr2: Group { orientation:'column', alignment:['fill','top'], \
                        gr21: Group { orientation:'row', alignment:['fill','top'], \
                            xRandomScl: Scrollbar { alignment:['fill','center'] }, \
                            xRandomEt: EditText { text:'0', characters:5, alignment:['right','center'] } \
                        }, \
                        gr22: Group { orientation:'row', alignment:['fill','top'], \
                            yRandomScl: Scrollbar { alignment:['fill','center'] }, \
                            yRandomEt: EditText { text:'0', characters:5, alignment:['right','center'] } \
                        }, \
                        gr23: Group { orientation:'row', alignment:['fill','top'], \
                            zRandomScl: Scrollbar { alignment:['fill','center'] }, \
                            zRandomEt: EditText { text:'0', characters:5, alignment:['right','center'] } \
                        }, \
                        gr24: Group { alignment:'fill', alignChildren:['left','top'], \
                            sliderRangeBtn: Button { text:'Slider Range' } \
                        } \
                    } \
                }, \
                pnl6: Panel { type:'tab', text:'Factor', orientation:'row', alignment:['fill','fill'], alignChildren:['right','top'], \
                    gr1: Group { orientation:'column', alignment:['left','top'], alignChildren:['right','center'], \
                           factorSt: StaticText { text:'Factor:', preferredSize:[-1,20] }, \
                        fooSt: StaticText { visible:false, text:'Offset:', preferredSize:[-1,20] }, \
                        fooSt: StaticText { visible:false, text:'Z Random:' } \
                    }, \
                    gr2: Group { orientation:'column', alignment:['fill','top'], \
                        gr21: Group { orientation:'row', alignment:['fill','top'], \
                            factorScl: Scrollbar { minvalue:0.75, maxvalue:1.5, value:1.0, stepdelta:0.01, jumpdelta:0.2, alignment:['fill','center'] }, \
                            factorEt: EditText { text:'1.0', characters:5, alignment:['right','center'] } \
                        }, \
                        gr22: Group { orientation:'row', alignment:['fill','top'], \
                            fooScl: Scrollbar { visible:false, alignment:['fill','center'] }, \
                            fooEt: EditText { visible:false, text:'0', characters:5, alignment:['right','center'] } \
                        }, \
                        gr23: Group { orientation:'row', alignment:['fill','top'], \
                            fooScl: Scrollbar { visible:false, minvalue:-100, alignment:['fill','center'] }, \
                            fooEt: EditText { visible:false, text:'0', characters:5, alignment:['right','center'] } \
                        }, \
                        gr24: Group { alignment:'fill', alignChildren:['left','top'], \
                            sliderRangeBtn: Button { text:'Slider Range' } \
                        } \
                    } \
                } \
            } \
        }"; 
        pal.gr = pal.add(res);
                
        pal.gr.pnl.pnl1.gr2.gr21.xOffsetEt.graphics.foregroundColor = 
        pal.gr.pnl.pnl1.gr2.gr22.yOffsetEt.graphics.foregroundColor = 
        pal.gr.pnl.pnl1.gr2.gr23.zOffsetEt.graphics.foregroundColor = 
        pal.gr.pnl.pnl2.gr2.gr21.xRotationEt.graphics.foregroundColor = 
        pal.gr.pnl.pnl2.gr2.gr22.yRotationEt.graphics.foregroundColor = 
        pal.gr.pnl.pnl2.gr2.gr23.zRotationEt.graphics.foregroundColor = 
        pal.gr.pnl.pnl3.gr2.gr21.xScaleEt.graphics.foregroundColor = 
        pal.gr.pnl.pnl3.gr2.gr22.yScaleEt.graphics.foregroundColor = 
        pal.gr.pnl.pnl3.gr2.gr23.opacityEt.graphics.foregroundColor = 
        pal.gr.pnl.pnl4.gr2.gr21.xRandomEt.graphics.foregroundColor = 
        pal.gr.pnl.pnl4.gr2.gr22.yRandomEt.graphics.foregroundColor = 
        pal.gr.pnl.pnl4.gr2.gr23.zRandomEt.graphics.foregroundColor = 
        pal.gr.pnl.pnl5.gr2.gr21.xRandomEt.graphics.foregroundColor = 
        pal.gr.pnl.pnl5.gr2.gr22.yRandomEt.graphics.foregroundColor = 
        pal.gr.pnl.pnl5.gr2.gr23.zRandomEt.graphics.foregroundColor = 
        pal.gr.pnl.pnl6.gr2.gr21.factorEt.graphics.foregroundColor = pal.graphics.newPen(pal.graphics.BrushType.SOLID_COLOR, [0,0,0], 1);

        pal.layout.layout(true);
        pal.gr.minimumSize = pal.gr.size;
        
        // event callbacks
        pal.onResizing = pal.onResize = function () 
        {
            this.layout.resize();
        };
                
        pal.gr.gr1.aboutBtn.onClick = function () 
        { 
            utils.createAboutDlg(distributeLayers.scriptAbout); 
        };
        
        pal.gr.pnl.pnl1.gr2.gr21.xOffsetScl.onChange = pal.gr.pnl.pnl1.gr2.gr21.xOffsetScl.onChanging = function () 
        {
            this.value = Math.floor(this.value);            
            this.parent.xOffsetEt.text = this.value;
            distributeLayers.offsetPosition(pal);
        };        

        pal.gr.pnl.pnl1.gr2.gr21.xOffsetEt.onChange = function () 
        {
            if (isNaN(this.text)) this.text = 0;
            this.parent.xOffsetScl.value = parseInt(this.text);
            distributeLayers.offsetPosition(pal);            
        };    
        
        pal.gr.pnl.pnl1.gr2.gr22.yOffsetScl.onChange = pal.gr.pnl.pnl1.gr2.gr22.yOffsetScl.onChanging = function () 
        {
            this.value = Math.floor(this.value);            
            this.parent.yOffsetEt.text = this.value;
            distributeLayers.offsetPosition(pal);
        };

        pal.gr.pnl.pnl1.gr2.gr22.yOffsetEt.onChange = function () 
        {
            if (isNaN(this.text)) this.text = 0;
            this.parent.yOffsetScl.value = parseInt(this.text);
            distributeLayers.offsetPosition(pal);            
        };
        
        pal.gr.pnl.pnl1.gr2.gr23.zOffsetScl.onChange = pal.gr.pnl.pnl1.gr2.gr23.zOffsetScl.onChanging = function () 
        {
            this.value = Math.floor(this.value);            
            this.parent.zOffsetEt.text = this.value;
            distributeLayers.offsetPosition(pal);
        };

        pal.gr.pnl.pnl1.gr2.gr23.zOffsetEt.onChange = function () 
        {
            if (isNaN(this.text)) this.text = 0;
            this.parent.zOffsetScl.value = parseInt(this.text);
            distributeLayers.offsetPosition(pal);            
        };

        pal.gr.pnl.pnl1.gr2.gr24.sliderRangeBtn.onClick = function () 
        {
            var minVal = parseInt(prompt("Minimum value:", pal.gr.pnl.pnl1.gr2.gr21.xOffsetScl.minvalue, "Slider Range"));
            var maxVal = parseInt(prompt("Maximum value:", pal.gr.pnl.pnl1.gr2.gr21.xOffsetScl.maxvalue, "Slider Range"));
            if (isNaN(minVal)) minVal = pal.gr.pnl.pnl1.gr2.gr21.xOffsetScl.minvalue;
            if (isNaN(maxVal)) maxVal = pal.gr.pnl.pnl1.gr2.gr21.xOffsetScl.maxvalue;
                        
            pal.gr.pnl.pnl1.gr2.gr21.xOffsetScl.minvalue = 
            pal.gr.pnl.pnl1.gr2.gr22.yOffsetScl.minvalue = 
            pal.gr.pnl.pnl1.gr2.gr23.zOffsetScl.minvalue = minVal;
            
            pal.gr.pnl.pnl1.gr2.gr21.xOffsetScl.maxvalue = 
            pal.gr.pnl.pnl1.gr2.gr22.yOffsetScl.maxvalue = 
            pal.gr.pnl.pnl1.gr2.gr23.zOffsetScl.maxvalue = maxVal;
        };
        
        pal.gr.pnl.pnl2.gr2.gr21.xRotationScl.onChange = pal.gr.pnl.pnl2.gr2.gr21.xRotationScl.onChanging = function () 
        {
            this.value = Math.floor(this.value);            
            this.parent.xRotationEt.text = this.value;
            distributeLayers.offsetRotation(pal, "x");
        };

        pal.gr.pnl.pnl2.gr2.gr21.xRotationEt.onChange = function () 
        {
            if (isNaN(this.text)) this.text = 0;
            this.parent.xRotationScl.value = parseInt(this.text);
            distributeLayers.offsetRotation(pal, "x");            
        };

        pal.gr.pnl.pnl2.gr2.gr22.yRotationScl.onChange = pal.gr.pnl.pnl2.gr2.gr22.yRotationScl.onChanging = function () 
        {
            this.value = Math.floor(this.value);            
            this.parent.yRotationEt.text = this.value;
            distributeLayers.offsetRotation(pal, "y");
        };

        pal.gr.pnl.pnl2.gr2.gr22.yRotationEt.onChange = function () 
        {
            if (isNaN(this.text)) this.text = 0;
            this.parent.yRotationScl.value = parseInt(this.text);
            distributeLayers.offsetRotation(pal, "y");            
        };
        
        pal.gr.pnl.pnl2.gr2.gr23.zRotationScl.onChange = pal.gr.pnl.pnl2.gr2.gr23.zRotationScl.onChanging = function () 
        {
            this.value = Math.floor(this.value);            
            this.parent.zRotationEt.text = this.value;
            distributeLayers.offsetRotation(pal, "z");
        };

        pal.gr.pnl.pnl2.gr2.gr23.zRotationEt.onChange = function () 
        {
            if (isNaN(this.text)) this.text = 0;
            this.parent.zRotationScl.value = parseInt(this.text);
            distributeLayers.offsetRotation(pal, "z");            
        };        
 
        pal.gr.pnl.pnl2.gr2.gr24.sliderRangeBtn.onClick = function () 
        {
            var minVal = parseInt(prompt("Minimum value:", pal.gr.pnl.pnl2.gr2.gr21.xRotationScl.minvalue, "Slider Range"));
            var maxVal = parseInt(prompt("Maximum value:", pal.gr.pnl.pnl2.gr2.gr21.xRotationScl.maxvalue, "Slider Range"));
            if (isNaN(minVal)) minVal = pal.gr.pnl.pnl2.gr2.gr21.xRotationScl.minvalue;
            if (isNaN(maxVal)) maxVal = pal.gr.pnl.pnl2.gr2.gr21.xRotationScl.maxvalue;
                        
            pal.gr.pnl.pnl2.gr2.gr21.xRotationScl.minvalue = 
            pal.gr.pnl.pnl2.gr2.gr22.yRotationScl.minvalue = 
            pal.gr.pnl.pnl2.gr2.gr23.zRotationScl.minvalue = minVal;
            
            pal.gr.pnl.pnl2.gr2.gr21.xRotationScl.maxvalue = 
            pal.gr.pnl.pnl2.gr2.gr22.yRotationScl.maxvalue = 
            pal.gr.pnl.pnl2.gr2.gr23.zRotationScl.maxvalue = maxVal;
        };

        pal.gr.pnl.pnl3.gr2.gr21.xScaleScl.onChange = pal.gr.pnl.pnl3.gr2.gr21.xScaleScl.onChanging = function () 
        {
            this.value = Math.floor(this.value);            
            this.parent.xScaleEt.text = this.value;
            distributeLayers.offsetScale(pal);
        };

        pal.gr.pnl.pnl3.gr2.gr21.xScaleEt.onChange = function () 
        {
            if (isNaN(this.text)) this.text = 0;
            this.parent.xScaleScl.value = parseInt(this.text);
            distributeLayers.offsetScale(pal);            
        };
        
        pal.gr.pnl.pnl3.gr2.gr22.yScaleScl.onChange = pal.gr.pnl.pnl3.gr2.gr22.yScaleScl.onChanging = function () 
        {
            this.value = Math.floor(this.value);            
            this.parent.yScaleEt.text = this.value;
            distributeLayers.offsetScale(pal);
        };

        pal.gr.pnl.pnl3.gr2.gr22.yScaleEt.onChange = function () 
        {
            if (isNaN(this.text)) this.text = 0;
            this.parent.yScaleScl.value = parseInt(this.text);
            distributeLayers.offsetScale(pal);            
        };        
        
        pal.gr.pnl.pnl3.gr2.gr23.opacityScl.onChange = pal.gr.pnl.pnl3.gr2.gr23.opacityScl.onChanging = function () 
        {
            this.value = Math.floor(this.value);            
            this.parent.opacityEt.text = this.value;
            distributeLayers.offsetOpacity(pal);
        };

        pal.gr.pnl.pnl3.gr2.gr23.opacityEt.onChange = function () 
        {
            if (isNaN(this.text)) this.text = 0;
            this.parent.opacityScl.value = parseInt(this.text);
            distributeLayers.offsetOpacity(pal);            
        };        

        pal.gr.pnl.pnl3.gr2.gr24.sliderRangeBtn.onClick = function () 
        {
            var minVal = parseInt(prompt("Minimum value:", pal.gr.pnl.pnl2.gr2.gr21.xScaleScl.minvalue, "Slider Range"));
            var maxVal = parseInt(prompt("Maximum value:", pal.gr.pnl.pnl2.gr2.gr21.xScaleScl.maxvalue, "Slider Range"));
            if (isNaN(minVal)) minVal = pal.gr.pnl.pnl2.gr2.gr21.xScaleScl.minvalue;
            if (isNaN(maxVal)) maxVal = pal.gr.pnl.pnl2.gr2.gr21.xScaleScl.maxvalue;            
            
            pal.gr.pnl.pnl3.gr2.gr21.xScaleScl.minvalue = 
            pal.gr.pnl.pnl3.gr2.gr22.yScaleScl.minvalue =             
            pal.gr.pnl.pnl3.gr2.gr23.opacityScl.minvalue = minVal;
            
            pal.gr.pnl.pnl3.gr2.gr21.xScaleScl.maxvalue =
            pal.gr.pnl.pnl3.gr2.gr22.yScaleScl.maxvalue =             
            pal.gr.pnl.pnl3.gr2.gr23.opacityScl.maxvalue = maxVal;
        };
                        
        pal.gr.pnl.pnl4.gr2.gr21.xRandomScl.onChange = pal.gr.pnl.pnl4.gr2.gr21.xRandomScl.onChanging = function () 
        {
            this.value = Math.floor(this.value);            
            this.parent.xRandomEt.text = this.value;
            distributeLayers.offsetPositionRandom(pal);
        };

        pal.gr.pnl.pnl4.gr2.gr21.xRandomEt.onChange = function () 
        {
            if (isNaN(this.text)) this.text = 0;
            this.parent.xRandomScl.value = parseInt(this.text);
            distributeLayers.offsetPositionRandom(pal);            
        };
        
        pal.gr.pnl.pnl4.gr2.gr22.yRandomScl.onChange = pal.gr.pnl.pnl4.gr2.gr22.yRandomScl.onChanging = function () 
        {
            this.value = Math.floor(this.value);            
            this.parent.yRandomEt.text = this.value;
            distributeLayers.offsetPositionRandom(pal);
        };

        pal.gr.pnl.pnl4.gr2.gr22.yRandomEt.onChange = function () 
        {
            if (isNaN(this.text)) this.text = 0;
            this.parent.yRandomScl.value = parseInt(this.text);
            distributeLayers.offsetPositionRandom(pal);            
        };         

        pal.gr.pnl.pnl4.gr2.gr23.zRandomScl.onChange = pal.gr.pnl.pnl4.gr2.gr23.zRandomScl.onChanging = function () 
        {
            this.value = Math.floor(this.value);            
            this.parent.zRandomEt.text = this.value;
            distributeLayers.offsetPositionRandom(pal);
        };

        pal.gr.pnl.pnl4.gr2.gr23.zRandomEt.onChange = function () 
        {
            if (isNaN(this.text)) this.text = 0;
            this.parent.zRandomScl.value = parseInt(this.text);
            distributeLayers.offsetPositionRandom(pal);            
        };
 
        pal.gr.pnl.pnl4.gr2.gr24.sliderRangeBtn.onClick = function () 
        {
            var minVal = parseInt(prompt("Minimum value:", pal.gr.pnl.pnl4.gr2.gr21.xRandomScl.minvalue, "Slider Range"));
            var maxVal = parseInt(prompt("Maximum value:", pal.gr.pnl.pnl4.gr2.gr21.xRandomScl.maxvalue, "Slider Range"));
            if (isNaN(minVal)) minVal = pal.gr.pnl.pnl4.gr2.gr21.xRandomScl.minvalue;
            if (isNaN(maxVal)) maxVal = pal.gr.pnl.pnl4.gr2.gr21.xRandomScl.maxvalue;
             
            pal.gr.pnl.pnl4.gr2.gr21.xRandomScl.minvalue = 
            pal.gr.pnl.pnl4.gr2.gr22.yRandomScl.minvalue = 
            pal.gr.pnl.pnl4.gr2.gr23.zRandomScl.minvalue = minVal;
            
            pal.gr.pnl.pnl4.gr2.gr21.xRandomScl.maxvalue = 
            pal.gr.pnl.pnl4.gr2.gr22.yRandomScl.maxvalue = 
            pal.gr.pnl.pnl4.gr2.gr23.zRandomScl.maxvalue = maxVal;
        };
        
        pal.gr.pnl.pnl5.gr2.gr21.xRandomScl.onChange = pal.gr.pnl.pnl5.gr2.gr21.xRandomScl.onChanging = function () 
        {
            this.value = Math.floor(this.value);            
            this.parent.xRandomEt.text = this.value;
            distributeLayers.offsetRotationRandom(pal, "x");
        };

        pal.gr.pnl.pnl5.gr2.gr21.xRandomEt.onChange = function () 
        {
            if (isNaN(this.text)) this.text = 0;
            this.parent.xRandomScl.value = parseInt(this.text);
            distributeLayers.offsetRotationRandom(pal, "x");            
        };
        
        pal.gr.pnl.pnl5.gr2.gr22.yRandomScl.onChange = pal.gr.pnl.pnl5.gr2.gr22.yRandomScl.onChanging = function () 
        {
            this.value = Math.floor(this.value);            
            this.parent.yRandomEt.text = this.value;
            distributeLayers.offsetRotationRandom(pal, "y");
        };

        pal.gr.pnl.pnl5.gr2.gr22.yRandomEt.onChange = function () 
        {
            if (isNaN(this.text)) this.text = 0;
            this.parent.yRandomScl.value = parseInt(this.text);
            distributeLayers.offsetRotationRandom(pal, "y");            
        };         

        pal.gr.pnl.pnl5.gr2.gr23.zRandomScl.onChange = pal.gr.pnl.pnl5.gr2.gr23.zRandomScl.onChanging = function () 
        {
            this.value = Math.floor(this.value);            
            this.parent.zRandomEt.text = this.value;
            distributeLayers.offsetRotationRandom(pal, "z");
        };

        pal.gr.pnl.pnl5.gr2.gr23.zRandomEt.onChange = function () 
        {
            if (isNaN(this.text)) this.text = 0;
            this.parent.zRandomScl.value = parseInt(this.text);
            distributeLayers.offsetRotationRandom(pal, "z");            
        };
 
        pal.gr.pnl.pnl5.gr2.gr24.sliderRangeBtn.onClick = function () 
        {
            var minVal = parseInt(prompt("Minimum value:", pal.gr.pnl.pnl5.gr2.gr21.xRandomScl.minvalue, "Slider Range"));
            var maxVal = parseInt(prompt("Maximum value:", pal.gr.pnl.pnl5.gr2.gr21.xRandomScl.maxvalue, "Slider Range"));
            if (isNaN(minVal)) minVal = pal.gr.pnl.pnl5.gr2.gr21.xRandomScl.minvalue;
            if (isNaN(maxVal)) maxVal = pal.gr.pnl.pnl5.gr2.gr21.xRandomScl.maxvalue;
                        
            pal.gr.pnl.pnl5.gr2.gr21.xRandomScl.minvalue = 
            pal.gr.pnl.pnl5.gr2.gr22.yRandomScl.minvalue = 
            pal.gr.pnl.pnl5.gr2.gr23.zRandomScl.minvalue = minVal;
            
            pal.gr.pnl.pnl5.gr2.gr21.xRandomScl.maxvalue = 
            pal.gr.pnl.pnl5.gr2.gr22.yRandomScl.maxvalue = 
            pal.gr.pnl.pnl5.gr2.gr23.zRandomScl.maxvalue = maxVal;
        };
                        
        pal.gr.pnl.pnl6.gr2.gr21.factorScl.onChange = pal.gr.pnl.pnl6.gr2.gr21.factorScl.onChanging = function () 
        {
            this.value = this.value.toFixed(2);            
            this.parent.factorEt.text = this.value;
            distributeLayers.offsetPosition(pal);
            distributeLayers.offsetRotation(pal, "x");
            distributeLayers.offsetRotation(pal, "y");
            distributeLayers.offsetRotation(pal, "z");
            distributeLayers.offsetScale(pal);
            distributeLayers.offsetOpacity(pal); 
        };        

        pal.gr.pnl.pnl6.gr2.gr21.factorEt.onChange = function () 
        {
            if (isNaN(this.text)) this.text = 1.0;
            this.parent.factorScl.value = parseFloat(this.text).toFixed(2);
            distributeLayers.offsetPosition(pal); 
            distributeLayers.offsetRotation(pal, "x");
            distributeLayers.offsetRotation(pal, "y");
            distributeLayers.offsetRotation(pal, "z");
            distributeLayers.offsetScale(pal);
            distributeLayers.offsetOpacity(pal); 
        };    

        pal.gr.pnl.pnl6.gr2.gr24.sliderRangeBtn.onClick = function () 
        {
            var minVal = parseInt(prompt("Minimum value:", pal.gr.pnl.pnl6.gr2.gr21.factorScl.minvalue, "Slider Range"));
            var maxVal = parseInt(prompt("Maximum value:", pal.gr.pnl.pnl6.gr2.gr21.factorScl.maxvalue, "Slider Range"));
            if (isNaN(minVal)) minVal = pal.gr.pnl.pnl6.gr2.gr21.factorScl.minvalue;
            if (isNaN(maxVal)) maxVal = pal.gr.pnl.pnl6.gr2.gr21.factorScl.maxvalue;
                        
            pal.gr.pnl.pnl6.gr2.gr21.factorScl.minvalue = minVal;            
            pal.gr.pnl.pnl6.gr2.gr21.factorScl.maxvalue = maxVal;
        };
                                                
        // show user interface
        if (pal instanceof Window)
        {
            pal.center();
            pal.show();
        }
        else
        {
            pal.layout.layout(true);
        }       
    };

    /**
     Determines whether the active item is a composition  
     @return True if the active item is not a composition, False otherwise
    */    
    this.checkActiveItem = function () 
    {
        var err = false;
        var comp = app.project.activeItem;
        if (!comp || !(comp instanceof CompItem))
        {
            err = true;
        }
        return err;
    }; 
    
    /**
     Offsets the position of selected layers along x/y/z axis 
     @param {Object} pal A palette or a dockable panel containing all user parameters         
    */    
    this.offsetPosition = function (pal)
    {
        try
        {
            var comp = app.project.activeItem;
            var err = this.noCompErr;
            if (this.checkActiveItem(comp)) throw(err);
                    
            var selLayers = comp.selectedLayers;
            var err = this.noLayersErr;
            if (selLayers.length < 1) throw(err);
            
            var xOffset = parseInt(pal.gr.pnl.pnl1.gr2.gr21.xOffsetEt.text);
            var yOffset = parseInt(pal.gr.pnl.pnl1.gr2.gr22.yOffsetEt.text);
            var zOffset = parseInt(pal.gr.pnl.pnl1.gr2.gr23.zOffsetEt.text);
            var factor = parseFloat(pal.gr.pnl.pnl6.gr2.gr21.factorEt.text);
            
            var prevOffset = [xOffset, yOffset, zOffset];
            
            app.beginUndoGroup(this.scriptTitle);
             
             var err = this.duringProcessErr;             
             try
             {
                 selLayers[0].threeDLayer = true;
                 
                 for (var i = 1; i < selLayers.length; i++)
                 {
                     selLayers[i].threeDLayer = true;
                                     
                    var prevP = selLayers[i-1].position;
                    var curP = selLayers[i].position;
                    
                    var curOffset = factor * prevOffset;                
                    var newP = prevP.valueAtTime(comp.time, false) + curOffset;                            
                    
                    curP.numKeys ? curP.setValueAtTime(comp.time, newP) : curP.setValue(newP);
                    
                    prevOffset = curOffset;
                 }
             }
             catch(e)
             {
                 throw(err);
             }
                  
            app.endUndoGroup();
        }
        catch(err)
        {
            utils.throwErr(err);
        }                
    };

    /**
     Offsets the x/y/z rotation of selected layers 
     @param {Object} pal A palette or a dockable panel containing all user parameters
     @param {String} axis A character that indicates the rotation axis ("x", "y" or "z")
    */    
    this.offsetRotation = function (pal, axis)
    {
        try
        {
            var comp = app.project.activeItem;
            var err = this.noCompErr;
            if (this.checkActiveItem(comp)) throw(err);
                    
            var selLayers = comp.selectedLayers;
            var err = this.noLayersErr;
            if (selLayers.length < 1) throw(err);
            
            var prevOffset = (axis == "x") ? parseInt(pal.gr.pnl.pnl2.gr2.gr21.xRotationEt.text) : 
                            ((axis == "y") ? parseInt(pal.gr.pnl.pnl2.gr2.gr22.yRotationEt.text) : parseInt(pal.gr.pnl.pnl2.gr2.gr23.zRotationEt.text));

            var factor = parseFloat(pal.gr.pnl.pnl6.gr2.gr21.factorEt.text);
            
            app.beginUndoGroup(this.scriptTitle);
             
             var err = this.duringProcessErr;             
             try
             {
                 if (!selLayers[0].threeDLayer) selLayers[0].threeDLayer = true;
                 
                 for (var i = 1; i < selLayers.length; i++)
                 {
                     if (!selLayers[i].threeDLayer) selLayers[i].threeDLayer = true;
                     
                     var prevR, curR;
                     if (axis == "x") 
                     {
                         prevR = selLayers[i-1].rotationX;
                         curR = selLayers[i].rotationX;
                     }
                     else if (axis == "y")
                     {
                         prevR = selLayers[i-1].rotationY;
                         curR = selLayers[i].rotationY;
                     }
                     else
                     {
                         prevR = selLayers[i-1].rotationZ;
                         curR = selLayers[i].rotationZ;
                     }
                    
                     var curOffset = factor * prevOffset;                               
                     var newR = prevR.valueAtTime(comp.time, false) + curOffset;                            
                    
                     curR.numKeys ? curR.setValueAtTime(comp.time, newR) : curR.setValue(newR);
                    
                     prevOffset = curOffset;                
                 }
             }
             catch(e)
             {
                 throw(err);
             }
                  
            app.endUndoGroup();
        }
        catch(err)
        {
            utils.throwErr(err);
        }                
    };

    /**
     Offsets the scale of selected layers 
     @param {Object} pal A palette or a dockable panel containing all user parameters
    */    
    this.offsetScale = function (pal)
    {
        try
        {
            var comp = app.project.activeItem;
            var err = this.noCompErr;
            if (this.checkActiveItem(comp)) throw(err);
                    
            var selLayers = comp.selectedLayers;
            var err = this.noLayersErr;
            if (selLayers.length < 1) throw(err);
            
            var xOffset = parseInt(pal.gr.pnl.pnl3.gr2.gr21.xScaleEt.text);
            var yOffset = parseInt(pal.gr.pnl.pnl3.gr2.gr22.yScaleEt.text);                            
            var factor = parseFloat(pal.gr.pnl.pnl6.gr2.gr21.factorEt.text);
            
            var prevOffset = [xOffset, yOffset, 0];
            
            app.beginUndoGroup(this.scriptTitle);
             
             var err = this.duringProcessErr;             
             try
             {
                 if (!selLayers[0].threeDLayer) selLayers[0].threeDLayer = true;
                 
                 for (var i = 1; i < selLayers.length; i++)
                 {
                     if (!selLayers[i].threeDLayer) selLayers[i].threeDLayer = true;
                     
                     var prevS = selLayers[i-1].scale;
                     var curS = selLayers[i].scale;
                    
                     var curOffset = factor * prevOffset;                               
                     var newS = prevS.valueAtTime(comp.time, false) + curOffset;                            
                    
                     curS.numKeys ? curS.setValueAtTime(comp.time, newS) : curS.setValue(newS);
                    
                     prevOffset = curOffset;                
                 }
             }
             catch(e)
             {
                 throw(err);
             }
                  
            app.endUndoGroup();
        }
        catch(err)
        {
            utils.throwErr(err);
        }                
    };
    
    /**
     Offsets the opacity of selected layers 
     @param {Object} pal A palette or a dockable panel containing all user parameters
    */    
    this.offsetOpacity = function (pal)
    {
        try
        {
            var comp = app.project.activeItem;
            var err = this.noCompErr;
            if (this.checkActiveItem(comp)) throw(err);
                    
            var selLayers = comp.selectedLayers;
            var err = this.noLayersErr;
            if (selLayers.length < 1) throw(err);
            
            var offset = parseInt(pal.gr.pnl.pnl3.gr2.gr23.opacityEt.text);                            
            var factor = parseFloat(pal.gr.pnl.pnl6.gr2.gr21.factorEt.text);
            
            var prevOffset = offset;
            
            app.beginUndoGroup(this.scriptTitle);
             
             var err = this.duringProcessErr;             
             try
             {
                 for (var i = 1; i < selLayers.length; i++)
                 {
                     var prevO = selLayers[i-1].opacity;
                     var curO = selLayers[i].opacity;
                    
                     var curOffset = factor * prevOffset;                               
                     var newO = Math.max(0, Math.min(100, prevO.valueAtTime(comp.time, false) - curOffset));                            
                    
                     curO.numKeys ? curO.setValueAtTime(comp.time, newO) : curO.setValue(newO);
                    
                     prevOffset = curOffset;                                
                 }
             }
             catch(e)
             {
                 throw(err);
             }
                  
            app.endUndoGroup();
        }
        catch(err)
        {
            utils.throwErr(err);
        }                
    };
    
    /**
     Randomly offsets the position of selected layers 
     @param {Object} pal A palette or a dockable panel containing all user parameters
    */    
    this.offsetPositionRandom = function (pal)
    {
        try
        {
            var comp = app.project.activeItem;
            var err = this.noCompErr;
            if (this.checkActiveItem(comp)) throw(err);
                    
            var selLayers = comp.selectedLayers;
            var err = this.noLayersErr;
            if (selLayers.length < 1) throw(err);
            
            var xRandom = parseInt(pal.gr.pnl.pnl4.gr2.gr21.xRandomEt.text);
            var yRandom = parseInt(pal.gr.pnl.pnl4.gr2.gr22.yRandomEt.text);
            var zRandom = parseInt(pal.gr.pnl.pnl4.gr2.gr23.zRandomEt.text);
            
            app.beginUndoGroup(this.scriptTitle);
             
            var err = this.duringProcessErr;             
            try
            {
                for (var i = 1; i < selLayers.length; i++)
                {
                    var curP = selLayers[i].position;
                    var offset = [-xRandom + 2*xRandom*Math.random(), -yRandom + 2*yRandom*Math.random(), -zRandom + 2*zRandom*Math.random()];
                    var newP = curP.valueAtTime(comp.time, false) + offset;                            
                    
                    curP.numKeys ? curP.setValueAtTime(comp.time, newP) : curP.setValue(newP);
                 }
             }
             catch(e)
             {
                 throw(err);
             }
                  
            app.endUndoGroup();
        }
        catch(err)
        {
            utils.throwErr(err);
        }                
    };

    /**
     Randomly offsets the x/y/z rotation of selected layers 
     @param {Object} pal A palette or a dockable panel containing all user parameters
     @param {String} axis A character that indicates the rotation axis ("x", "y" or "z")
    */    
    this.offsetRotationRandom = function (pal, axis)
    {
        try
        {
            var comp = app.project.activeItem;
            var err = this.noCompErr;
            if (this.checkActiveItem(comp)) throw(err);
                    
            var selLayers = comp.selectedLayers;
            var err = this.noLayersErr;
            if (selLayers.length < 1) throw(err);
            
            var xRandom = parseInt(pal.gr.pnl.pnl5.gr2.gr21.xRandomEt.text);
            var yRandom = parseInt(pal.gr.pnl.pnl5.gr2.gr22.yRandomEt.text);
            var zRandom = parseInt(pal.gr.pnl.pnl5.gr2.gr23.zRandomEt.text);
                                             
            app.beginUndoGroup(this.scriptTitle);
             
             var err = this.duringProcessErr;             
             try
             {
                 for (var i = 1; i < selLayers.length; i++)
                 {
                     var curR = (axis == "x") ? selLayers[i].rotationX : ((axis == "y") ? selLayers[i].rotationY : selLayers[i].rotationZ);
                    
                     var offset = (axis == "x") ? -xRandom + 2*xRandom*Math.random() : 
                                 ((axis == "y") ? -yRandom + 2*yRandom*Math.random() : -zRandom + 2*zRandom*Math.random());
                                
                     var newR = curR.valueAtTime(comp.time, false) + offset;                            
                    
                     curR.numKeys ? curR.setValueAtTime(comp.time, newR) : curR.setValue(newR);
                 }
             }
             catch(e)
             {
                 throw(err);
             }
                  
            app.endUndoGroup();
        }
        catch(err)
        {
            utils.throwErr(err);
        }                
    };    
            
    /**
     Runs the script  
     @param {Object} thisObj A Panel object if the script is launched from the Window menu, null otherwise
    */
    this.run = function (thisObj) 
    {
        if (parseFloat(app.version) < parseFloat(this.scriptMinSupportVersion))
        {
            utils.throwErr(this.requirementErr);
        }
        else
        {
            this.buildUI(thisObj);
        }    
    };
}


/**
 This class provides some utility functions used by DistributeLayers
 @class Some utility functions grouped in a class
*/
function DistributeLayersUtils()
{
    /**
     String localization function: english and french languages are supported
     @param {Object} str A localization object containing the localized versions of a string    
     @return Appropriate localized version of str
    */    
    this.loc = function (str)
    {
        return app.language == Language.FRENCH ? str.fr : str.en;
    };

    /**
     Displays a window containg a localized error message
     @param {Object} err A localization object containing the localized versions of an error message
    */    
    this.throwErr = function (err)
    {
        var wndTitle = $.fileName.substring($.fileName.lastIndexOf("/")+1, $.fileName.lastIndexOf("."));
        Window.alert("Script error:\r" + this.loc(err), wndTitle, true);
    };            

    /**
     Displays a customized window containg the About text
     @param {String} aboutStr The text to display
    */
    this.createAboutDlg = function (aboutStr)
    {        
        eval(unescape('%20%20%20%20%20%20%20%20%2F%2A%2A%20%0A%20%20%20%20%20%20%20%20%20%44%72%61%77%20%73%6F%6D%65%20%72%61%6E%64%6F%6D%20%72%65%63%74%61%6E%67%6C%65%73%20%28%70%6F%73%69%74%69%6F%6E%2C%20%73%69%7A%65%2C%20%63%6F%6C%6F%72%2C%20%61%6C%70%68%61%29%20%6F%6E%20%74%68%65%20%77%69%6E%64%6F%77%20%62%61%63%6B%67%72%6F%75%6E%64%0A%20%20%20%20%20%20%20%20%20%40%69%67%6E%6F%72%65%20%0A%20%20%20%20%20%20%20%20%2A%2F%0A%20%20%20%20%20%20%20%20%66%75%6E%63%74%69%6F%6E%20%61%64%64%4E%61%62%73%63%72%69%70%74%73%42%61%63%6B%67%72%6F%75%6E%64%53%69%67%6E%61%74%75%72%65%28%77%6E%64%29%0A%20%20%20%20%20%20%20%20%7B%0A%20%20%20%20%20%20%20%20%20%20%20%20%76%61%72%20%6E%75%6D%52%65%63%74%20%3D%20%32%34%3B%0A%20%20%20%20%20%20%20%20%20%20%20%20%76%61%72%20%6D%69%6E%4F%70%61%63%69%74%79%20%3D%20%30%2E%30%35%3B%0A%20%20%20%20%20%20%20%20%20%20%20%20%76%61%72%20%6D%61%78%4F%70%61%63%69%74%79%20%3D%20%30%2E%31%35%3B%0A%20%20%20%20%20%20%20%20%0A%20%20%20%20%20%20%20%20%20%20%20%20%76%61%72%20%6C%65%66%74%45%64%67%65%20%3D%20%30%3B%0A%20%20%20%20%20%20%20%20%20%20%20%20%76%61%72%20%74%6F%70%45%64%67%65%20%3D%20%30%3B%20%20%20%20%0A%20%20%20%20%20%20%20%20%20%20%20%20%76%61%72%20%72%69%67%68%74%45%64%67%65%20%3D%20%77%6E%64%2E%77%69%6E%64%6F%77%42%6F%75%6E%64%73%2E%77%69%64%74%68%3B%0A%20%20%20%20%20%20%20%20%20%20%20%20%76%61%72%20%62%6F%74%74%6F%6D%45%64%67%65%20%3D%20%77%6E%64%2E%77%69%6E%64%6F%77%42%6F%75%6E%64%73%2E%68%65%69%67%68%74%3B%0A%20%20%20%20%20%20%20%20%0A%20%20%20%20%20%20%20%20%20%20%20%20%66%6F%72%20%28%76%61%72%20%69%20%3D%20%30%20%3B%20%69%20%3C%20%6E%75%6D%52%65%63%74%3B%20%69%2B%2B%29%0A%20%20%20%20%20%20%20%20%20%20%20%20%7B%0A%20%20%20%20%20%20%20%20%20%20%20%20%20%20%20%20%76%61%72%20%78%4C%6F%63%20%3D%20%31%30%20%2B%20%28%72%69%67%68%74%45%64%67%65%20%2D%20%32%30%29%20%2A%20%4D%61%74%68%2E%72%61%6E%64%6F%6D%28%29%3B%0A%20%20%20%20%20%20%20%20%20%20%20%20%20%20%20%20%76%61%72%20%79%4C%6F%63%20%3D%20%31%30%20%2B%20%28%62%6F%74%74%6F%6D%45%64%67%65%20%2D%20%32%30%29%20%2A%20%4D%61%74%68%2E%72%61%6E%64%6F%6D%28%29%3B%0A%20%20%20%20%20%20%20%20%20%20%20%20%20%20%20%20%76%61%72%20%77%69%64%74%68%20%3D%20%35%20%2B%20%31%35%20%2A%20%4D%61%74%68%2E%72%61%6E%64%6F%6D%28%29%3B%0A%20%20%20%20%20%20%20%20%20%20%20%20%20%20%20%20%76%61%72%20%68%65%69%67%68%74%20%3D%20%35%20%2B%20%31%35%20%2A%20%4D%61%74%68%2E%72%61%6E%64%6F%6D%28%29%3B%20%20%20%20%20%0A%20%20%20%20%20%20%20%20%20%20%20%20%20%20%20%20%76%61%72%20%62%6F%72%64%65%72%57%69%64%74%68%20%3D%20%31%20%2B%20%34%20%2A%20%4D%61%74%68%2E%72%61%6E%64%6F%6D%28%29%3B%0A%20%20%20%20%20%20%20%20%20%20%20%20%20%20%20%20%76%61%72%20%62%6F%72%64%65%72%43%6F%6C%6F%72%20%3D%20%5B%4D%61%74%68%2E%72%61%6E%64%6F%6D%28%29%2C%20%4D%61%74%68%2E%72%61%6E%64%6F%6D%28%29%2C%20%4D%61%74%68%2E%72%61%6E%64%6F%6D%28%29%2C%20%6D%69%6E%4F%70%61%63%69%74%79%20%2B%20%28%6D%61%78%4F%70%61%63%69%74%79%20%2D%20%6D%69%6E%4F%70%61%63%69%74%79%29%20%2A%20%4D%61%74%68%2E%72%61%6E%64%6F%6D%28%29%5D%3B%0A%20%20%20%20%20%20%20%20%20%20%20%20%0A%20%20%20%20%20%20%20%20%20%20%20%20%20%20%20%20%76%61%72%20%63%6F%6C%6F%72%42%72%75%73%68%20%3D%20%77%6E%64%2E%67%72%61%70%68%69%63%73%2E%6E%65%77%42%72%75%73%68%28%77%6E%64%2E%67%72%61%70%68%69%63%73%2E%42%72%75%73%68%54%79%70%65%2E%53%4F%4C%49%44%5F%43%4F%4C%4F%52%2C%20%62%6F%72%64%65%72%43%6F%6C%6F%72%29%3B%0A%20%20%20%20%20%20%20%20%20%20%20%20%0A%20%20%20%20%20%20%20%20%20%20%20%20%20%20%20%20%77%6E%64%2E%67%31%20%3D%20%77%6E%64%2E%61%64%64%28%22%67%72%6F%75%70%22%2C%20%5B%78%4C%6F%63%2C%20%79%4C%6F%63%2C%20%78%4C%6F%63%20%2B%20%77%69%64%74%68%2C%20%79%4C%6F%63%20%2B%20%62%6F%72%64%65%72%57%69%64%74%68%5D%29%3B%0A%20%20%20%20%20%20%20%20%20%20%20%20%20%20%20%20%77%6E%64%2E%67%32%20%3D%20%77%6E%64%2E%61%64%64%28%22%67%72%6F%75%70%22%2C%20%5B%78%4C%6F%63%2C%20%79%4C%6F%63%20%2B%20%68%65%69%67%68%74%20%2D%20%62%6F%72%64%65%72%57%69%64%74%68%2C%20%78%4C%6F%63%20%2B%20%77%69%64%74%68%2C%20%79%4C%6F%63%20%2B%20%68%65%69%67%68%74%5D%29%3B%0A%20%20%20%20%20%20%20%20%20%20%20%20%20%20%20%20%77%6E%64%2E%67%33%20%3D%20%77%6E%64%2E%61%64%64%28%22%67%72%6F%75%70%22%2C%20%5B%78%4C%6F%63%2C%20%79%4C%6F%63%20%2B%20%62%6F%72%64%65%72%57%69%64%74%68%2C%20%78%4C%6F%63%20%2B%20%62%6F%72%64%65%72%57%69%64%74%68%2C%20%79%4C%6F%63%20%2B%20%68%65%69%67%68%74%20%2D%20%62%6F%72%64%65%72%57%69%64%74%68%5D%29%3B%0A%20%20%20%20%20%20%20%20%20%20%20%20%20%20%20%20%77%6E%64%2E%67%34%20%3D%20%77%6E%64%2E%61%64%64%28%22%67%72%6F%75%70%22%2C%20%5B%78%4C%6F%63%20%2B%20%77%69%64%74%68%20%2D%20%62%6F%72%64%65%72%57%69%64%74%68%2C%20%79%4C%6F%63%20%2B%20%62%6F%72%64%65%72%57%69%64%74%68%2C%20%78%4C%6F%63%20%2B%20%77%69%64%74%68%2C%20%79%4C%6F%63%20%2B%20%68%65%69%67%68%74%20%2D%20%62%6F%72%64%65%72%57%69%64%74%68%5D%29%3B%0A%20%20%20%20%20%20%20%20%0A%20%20%20%20%20%20%20%20%20%20%20%20%20%20%20%20%77%6E%64%2E%67%31%2E%67%72%61%70%68%69%63%73%2E%62%61%63%6B%67%72%6F%75%6E%64%43%6F%6C%6F%72%20%3D%20%0A%20%20%20%20%20%20%20%20%20%20%20%20%20%20%20%20%77%6E%64%2E%67%32%2E%67%72%61%70%68%69%63%73%2E%62%61%63%6B%67%72%6F%75%6E%64%43%6F%6C%6F%72%20%3D%20%0A%20%20%20%20%20%20%20%20%20%20%20%20%20%20%20%20%77%6E%64%2E%67%33%2E%67%72%61%70%68%69%63%73%2E%62%61%63%6B%67%72%6F%75%6E%64%43%6F%6C%6F%72%20%3D%20%0A%20%20%20%20%20%20%20%20%20%20%20%20%20%20%20%20%77%6E%64%2E%67%34%2E%67%72%61%70%68%69%63%73%2E%62%61%63%6B%67%72%6F%75%6E%64%43%6F%6C%6F%72%20%3D%20%63%6F%6C%6F%72%42%72%75%73%68%3B%0A%20%20%20%20%20%20%20%20%20%20%20%20%7D%20%20%20%20%0A%20%20%20%20%20%20%20%20%7D%0A%20%20%20%20%20%20%20%20%0A%20%20%20%20%20%20%20%20%76%61%72%20%64%6C%67%20%3D%20%6E%65%77%20%57%69%6E%64%6F%77%28%22%64%69%61%6C%6F%67%22%2C%20%22%41%62%6F%75%74%22%29%3B%0A%20%20%20%20%20%20%20%20%0A%20%20%20%20%20%20%20%20%2F%2F%20%70%61%6E%65%6C%20%62%6F%72%64%65%72%53%74%79%6C%65%3A%20%6F%6E%65%20%6F%66%20%62%6C%61%63%6B%2C%20%65%74%63%68%65%64%2C%20%67%72%61%79%2C%20%72%61%69%73%65%64%2C%20%73%75%6E%6B%65%6E%2E%20%44%65%66%61%75%6C%74%20%69%73%20%65%74%63%68%65%64%2E%0A%20%20%20%20%20%20%20%20%2F%2F%20%72%65%73%6F%75%72%63%65%20%73%70%65%63%69%66%69%63%61%74%69%6F%6E%73%0A%20%20%20%20%20%20%20%20%76%61%72%20%72%65%73%20%3D%0A%20%20%20%20%20%20%20%20%22%67%72%6F%75%70%20%7B%20%6F%72%69%65%6E%74%61%74%69%6F%6E%3A%27%63%6F%6C%75%6D%6E%27%2C%20%61%6C%69%67%6E%6D%65%6E%74%3A%5B%27%66%69%6C%6C%27%2C%27%66%69%6C%6C%27%5D%2C%20%5C%0A%20%20%20%20%20%20%20%20%20%20%20%20%61%62%6F%75%74%50%6E%6C%3A%20%50%61%6E%65%6C%20%7B%20%70%72%6F%70%65%72%74%69%65%73%3A%7B%20%62%6F%72%64%65%72%53%74%79%6C%65%3A%27%73%75%6E%6B%65%6E%27%20%7D%2C%20%5C%0A%20%20%20%20%20%20%20%20%20%20%20%20%20%20%20%20%61%62%6F%75%74%45%74%3A%20%45%64%69%74%54%65%78%74%20%7B%20%74%65%78%74%3A%27%22%20%2B%20%74%68%69%73%2E%6C%6F%63%28%61%62%6F%75%74%53%74%72%29%20%2B%20%22%27%2C%20%70%72%6F%70%65%72%74%69%65%73%3A%7B%6D%75%6C%74%69%6C%69%6E%65%3A%74%72%75%65%7D%2C%20%70%72%65%66%65%72%72%65%64%53%69%7A%65%3A%5B%32%38%30%2C%31%35%30%5D%2C%20%61%6C%69%67%6E%6D%65%6E%74%3A%5B%27%72%69%67%68%74%27%2C%27%63%65%6E%74%65%72%27%5D%20%7D%20%5C%0A%20%20%20%20%20%20%20%20%20%20%20%20%7D%2C%20%5C%0A%20%20%20%20%20%20%20%20%20%20%20%20%62%74%6E%73%47%72%3A%20%47%72%6F%75%70%20%7B%20%61%6C%69%67%6E%6D%65%6E%74%3A%5B%27%66%69%6C%6C%27%2C%27%66%69%6C%6C%27%5D%2C%20%5C%0A%20%20%20%20%20%20%20%20%20%20%20%20%20%20%20%20%76%69%73%69%74%42%74%6E%3A%20%42%75%74%74%6F%6E%20%7B%20%74%65%78%74%3A%27%56%69%73%69%74%20%48%6F%6D%65%70%61%67%65%27%2C%20%61%6C%69%67%6E%6D%65%6E%74%3A%5B%27%6C%65%66%74%27%2C%27%63%65%6E%74%65%72%27%5D%20%7D%2C%20%5C%0A%20%20%20%20%20%20%20%20%20%20%20%20%20%20%20%20%6F%6B%42%74%6E%3A%20%42%75%74%74%6F%6E%20%7B%20%74%65%78%74%3A%27%4F%6B%27%2C%20%61%6C%69%67%6E%6D%65%6E%74%3A%5B%27%72%69%67%68%74%27%2C%27%63%65%6E%74%65%72%27%5D%20%7D%20%5C%0A%20%20%20%20%20%20%20%20%20%20%20%20%7D%20%5C%0A%20%20%20%20%20%20%20%20%7D%22%3B%20%0A%20%20%20%20%20%20%20%20%64%6C%67%2E%67%72%20%3D%20%64%6C%67%2E%61%64%64%28%72%65%73%29%3B%0A%20%20%20%20%20%20%20%20%0A%20%20%20%20%20%20%20%20%2F%2F%20%6F%6E%20%4D%61%63%20%77%65%20%63%61%6E%20%64%69%73%61%62%6C%65%20%65%64%69%74%20%74%65%78%74%20%77%68%69%6C%65%20%61%6C%6C%6F%77%69%6E%67%20%73%63%72%6F%6C%6C%69%6E%67%2C%20%6F%6E%20%57%69%6E%64%6F%77%73%20%77%65%20%63%61%6E%27%74%0A%20%20%20%20%20%20%20%20%64%6C%67%2E%67%72%2E%61%62%6F%75%74%50%6E%6C%2E%61%62%6F%75%74%45%74%2E%65%6E%61%62%6C%65%64%20%3D%20%28%24%2E%6F%73%2E%69%6E%64%65%78%4F%66%28%22%57%69%6E%22%29%20%21%3D%20%2D%31%29%3B%0A%20%20%20%20%20%20%20%20%20%20%20%20%20%20%20%20%0A%20%20%20%20%20%20%20%20%2F%2F%20%64%72%61%77%20%72%61%6E%64%6F%6D%20%62%61%63%6B%67%72%6F%75%6E%64%20%63%6F%6C%6F%72%20%28%67%72%61%79%73%63%61%6C%65%29%0A%20%20%20%20%20%20%20%20%69%66%20%28%70%61%72%73%65%46%6C%6F%61%74%28%61%70%70%2E%76%65%72%73%69%6F%6E%29%20%3E%3D%20%39%2E%30%29%20%2F%2F%20%43%53%34%20%6F%72%20%6C%61%74%65%72%0A%20%20%20%20%20%20%20%20%7B%0A%20%20%20%20%20%20%20%20%20%20%20%20%76%61%72%20%77%68%69%74%65%42%72%75%73%68%20%3D%20%64%6C%67%2E%67%72%61%70%68%69%63%73%2E%6E%65%77%42%72%75%73%68%28%64%6C%67%2E%67%72%61%70%68%69%63%73%2E%42%72%75%73%68%54%79%70%65%2E%53%4F%4C%49%44%5F%43%4F%4C%4F%52%2C%20%5B%31%2C%20%31%2C%20%31%2C%20%31%5D%29%3B%0A%20%20%20%20%20%20%20%20%20%20%20%20%76%61%72%20%72%61%6E%64%20%3D%20%4D%61%74%68%2E%72%61%6E%64%6F%6D%28%29%3B%0A%20%20%20%20%20%20%20%20%20%20%20%20%76%61%72%20%62%67%42%72%75%73%68%20%3D%20%64%6C%67%2E%67%72%61%70%68%69%63%73%2E%6E%65%77%42%72%75%73%68%28%64%6C%67%2E%67%72%61%70%68%69%63%73%2E%42%72%75%73%68%54%79%70%65%2E%53%4F%4C%49%44%5F%43%4F%4C%4F%52%2C%20%5B%72%61%6E%64%2C%20%72%61%6E%64%2C%20%72%61%6E%64%2C%20%31%5D%29%3B%0A%20%20%20%20%20%20%20%20%20%20%20%20%64%6C%67%2E%67%72%61%70%68%69%63%73%2E%62%61%63%6B%67%72%6F%75%6E%64%43%6F%6C%6F%72%20%3D%20%62%67%42%72%75%73%68%3B%0A%20%20%20%20%20%20%20%20%20%20%20%20%64%6C%67%2E%67%72%2E%61%62%6F%75%74%50%6E%6C%2E%67%72%61%70%68%69%63%73%2E%62%61%63%6B%67%72%6F%75%6E%64%43%6F%6C%6F%72%20%3D%20%77%68%69%74%65%42%72%75%73%68%3B%0A%20%20%20%20%20%20%20%20%0A%20%20%20%20%20%20%20%20%20%20%20%20%64%6C%67%2E%6C%61%79%6F%75%74%2E%6C%61%79%6F%75%74%28%74%72%75%65%29%3B%20%2F%2F%20%74%6F%20%67%65%74%20%77%69%6E%64%6F%77%20%62%6F%75%6E%64%73%0A%20%20%20%20%20%20%20%20%20%20%20%20%0A%20%20%20%20%20%20%20%20%20%20%20%20%61%64%64%4E%61%62%73%63%72%69%70%74%73%42%61%63%6B%67%72%6F%75%6E%64%53%69%67%6E%61%74%75%72%65%28%64%6C%67%29%3B%0A%20%20%20%20%20%20%20%20%7D%0A%20%20%20%20%20%20%20%20%0A%20%20%20%20%20%20%20%20%64%6C%67%2E%67%72%2E%62%74%6E%73%47%72%2E%6F%6B%42%74%6E%2E%6F%6E%43%6C%69%63%6B%20%3D%20%66%75%6E%63%74%69%6F%6E%20%28%29%20%7B%20%64%6C%67%2E%63%6C%6F%73%65%28%29%3B%20%7D%3B%0A%20%20%20%20%20%20%20%20%20%20%20%20%20%20%20%20%0A%20%20%20%20%20%20%20%20%2F%2F%20%6F%70%65%6E%20%68%6F%6D%65%70%61%67%65%20%75%72%6C%0A%20%20%20%20%20%20%20%20%64%6C%67%2E%67%72%2E%62%74%6E%73%47%72%2E%76%69%73%69%74%42%74%6E%2E%6F%6E%43%6C%69%63%6B%20%3D%20%66%75%6E%63%74%69%6F%6E%20%28%29%20%0A%20%20%20%20%20%20%20%20%7B%0A%20%20%20%20%20%20%20%20%20%20%20%20%76%61%72%20%63%6D%64%20%3D%20%22%22%3B%0A%20%20%20%20%20%20%20%20%20%20%20%20%76%61%72%20%75%72%6C%20%3D%20%22%68%74%74%70%3A%2F%2F%77%77%77%2E%6E%61%62%73%63%72%69%70%74%73%2E%63%6F%6D%2F%22%3B%0A%20%20%20%20%20%20%20%20%20%20%20%20%0A%20%20%20%20%20%20%20%20%20%20%20%20%69%66%20%28%24%2E%6F%73%2E%69%6E%64%65%78%4F%66%28%22%57%69%6E%22%29%20%21%3D%20%2D%31%29%0A%20%20%20%20%20%20%20%20%20%20%20%20%7B%0A%20%20%20%20%20%20%20%20%20%20%20%20%20%20%20%20%69%66%20%28%46%69%6C%65%28%22%43%3A%2F%50%72%6F%67%72%61%6D%20%46%69%6C%65%73%2F%4D%6F%7A%69%6C%6C%61%20%46%69%72%65%66%6F%78%2F%66%69%72%65%66%6F%78%2E%65%78%65%22%29%2E%65%78%69%73%74%73%29%0A%20%20%20%20%20%20%20%20%20%20%20%20%20%20%20%20%7B%0A%09%09%09%09%09%63%6D%64%20%2B%3D%20%22%43%3A%2F%50%72%6F%67%72%61%6D%20%46%69%6C%65%73%2F%4D%6F%7A%69%6C%6C%61%20%46%69%72%65%66%6F%78%2F%66%69%72%65%66%6F%78%2E%65%78%65%20%22%20%2B%20%75%72%6C%3B%0A%20%20%20%20%20%20%20%20%20%20%20%20%20%20%20%20%7D%0A%20%20%20%20%20%20%20%20%20%20%20%20%20%20%20%20%65%6C%73%65%0A%20%20%20%20%20%20%20%20%20%20%20%20%20%20%20%20%7B%0A%20%20%20%20%20%20%20%20%20%20%20%20%20%20%20%20%09%63%6D%64%20%2B%3D%20%22%43%3A%2F%50%72%6F%67%72%61%6D%20%46%69%6C%65%73%2F%49%6E%74%65%72%6E%65%74%20%45%78%70%6C%6F%72%65%72%2F%69%65%78%70%6C%6F%72%65%2E%65%78%65%20%22%20%2B%20%75%72%6C%3B%0A%09%09%09%09%7D%0A%20%20%20%20%20%20%20%20%20%20%20%20%7D%0A%20%20%20%20%20%20%20%20%20%20%20%20%65%6C%73%65%0A%20%20%20%20%20%20%20%20%20%20%20%20%7B%0A%20%20%20%20%20%20%20%20%20%20%20%20%09%63%6D%64%20%2B%3D%20%22%6F%70%65%6E%20%5C%22%22%20%2B%20%75%72%6C%20%2B%20%22%5C%22%22%3B%0A%20%20%20%20%20%20%20%20%20%20%20%20%7D%0A%20%20%20%20%20%20%20%20%20%20%20%20%0A%20%20%20%20%20%20%20%20%20%20%20%20%74%72%79%0A%20%20%20%20%20%20%20%20%20%20%20%20%7B%0A%09%09%09%09%73%79%73%74%65%6D%2E%63%61%6C%6C%53%79%73%74%65%6D%28%63%6D%64%29%3B%0A%09%09%09%7D%0A%09%09%09%63%61%74%63%68%28%65%29%0A%09%09%09%7B%0A%09%09%09%09%61%6C%65%72%74%28%65%29%3B%0A%09%09%09%7D%0A%20%20%20%20%20%20%20%20%7D%0A%20%20%20%20%20%20%20%20%0A%20%20%20%20%20%20%20%20%64%6C%67%2E%63%65%6E%74%65%72%28%29%3B%0A%20%20%20%20%20%20%20%20%64%6C%67%2E%73%68%6F%77%28%29%3B'));
    };
}


/**
 Creates an instance of the main class and run it
*/
new DistributeLayers().run(this);
